@@ -38,6 +38,7 @@ module Ast_414 = Ast_414
 module Ast_500 = Ast_500
 module Ast_501 = Ast_501
 module Ast_502 = Ast_502
+module Ast_503 = Ast_503
 (*$*)
 
 (* Manual migration between versions *)
@@ -76,6 +77,8 @@ module Migrate_500_501 = Migrate_500_501
 module Migrate_501_500 = Migrate_501_500
 module Migrate_501_502 = Migrate_501_502
 module Migrate_502_501 = Migrate_502_501
+module Migrate_502_503 = Migrate_502_503
+module Migrate_503_502 = Migrate_503_502
 (*$*)
 
 (* Compiler modules *)
@@ -86,6 +89,12 @@ module Location = Location
 module Longident = Longident
 module Parse = Parse
 module Pprintast = Pprintast
+module Compiler_pprintast = struct
+  include Ocaml_common.Pprintast
+
+  let structure_item fmt t = structure fmt [t]
+  let signature_item fmt t = signature fmt [t]
+end
 
 let init_error_reporting_style_using_env_vars () =
   (*IF_AT_LEAST 408 Ocaml_common.Compmisc.read_clflags_from_env () *)
