@@ -80,6 +80,10 @@ module Rule = struct
       | Attr_sig_type_ext : (signature_item, type_extension) Attr_inline.t t
       | Attr_str_exception : (structure_item, type_exception) Attr_inline.t t
       | Attr_sig_exception : (signature_item, type_exception) Attr_inline.t t
+      | Attr_str_class_type_decl
+          : (structure_item, class_type_declaration) Attr_group_inline.t t
+      | Attr_sig_class_type_decl
+          : (signature_item, class_type_declaration) Attr_group_inline.t t
 
     type (_, _) equality = Eq : ('a, 'a) equality | Ne : (_, _) equality
 
@@ -97,6 +101,8 @@ module Rule = struct
       | Attr_sig_exception, Attr_sig_exception -> Eq
       | Attr_str_module_type_decl, Attr_str_module_type_decl -> Eq
       | Attr_sig_module_type_decl, Attr_sig_module_type_decl -> Eq
+      | Attr_str_class_type_decl, Attr_str_class_type_decl -> Eq
+      | Attr_sig_class_type_decl, Attr_sig_class_type_decl -> Eq
       | _ -> Ne
   end
 
@@ -130,6 +136,9 @@ module Rule = struct
   let special_function id f =
     T (Special_function, { name = id; ident = Longident.parse id; expand = f })
 
+  let special_function' ident f =
+    T (Special_function, { name = Longident.name ident; ident; expand = f })
+
   let constant kind suffix expand = T (Constant, { suffix; kind; expand })
 
   let attr_str_type_decl attribute expand =
@@ -156,6 +165,12 @@ module Rule = struct
   let attr_sig_exception attribute expand =
     T (Attr_sig_exception, T { attribute; expand; expect = false })
 
+  let attr_str_class_type_decl attribute expand =
+    T (Attr_str_class_type_decl, T { attribute; expand; expect = false })
+
+  let attr_sig_class_type_decl attribute expand =
+    T (Attr_sig_class_type_decl, T { attribute; expand; expect = false })
+
   let attr_str_type_decl_expect attribute expand =
     T (Attr_str_type_decl, T { attribute; expand; expect = true })
 
@@ -179,6 +194,12 @@ module Rule = struct
 
   let attr_sig_exception_expect attribute expand =
     T (Attr_sig_exception, T { attribute; expand; expect = true })
+
+  let attr_str_class_type_decl_expect attribute expand =
+    T (Attr_str_class_type_decl, T { attribute; expand; expect = true })
+
+  let attr_sig_class_type_decl_expect attribute expand =
+    T (Attr_sig_class_type_decl, T { attribute; expand; expect = true })
 end
 
 module Generated_code_hook = struct
@@ -197,6 +218,48 @@ module Generated_code_hook = struct
     | _ -> t.f context { loc with loc_start = loc.loc_end } x
 end
 
+(* Used to insert error extensions *)
+let wrap_extension : type a. loc:Location.t -> a EC.t -> a -> extension -> a =
+ fun ~loc t original_node extension ->
+  (* Prefixing constructors with the module path is necessary for OCaml < 4.07,
+     see https://github.com/ocaml/ocaml/issues/6852 *)
+  match t with
+  | EC.Class_expr -> Ast_builder.Default.pcl_extension ~loc extension
+  | EC.Class_field -> Ast_builder.Default.pcf_extension ~loc extension
+  | EC.Class_type -> Ast_builder.Default.pcty_extension ~loc extension
+  | EC.Class_type_field -> Ast_builder.Default.pctf_extension ~loc extension
+  | EC.Core_type -> Ast_builder.Default.ptyp_extension ~loc extension
+  | EC.Expression -> Ast_builder.Default.pexp_extension ~loc extension
+  | EC.Module_expr -> Ast_builder.Default.pmod_extension ~loc extension
+  | EC.Module_type -> Ast_builder.Default.pmty_extension ~loc extension
+  | EC.Pattern -> Ast_builder.Default.ppat_extension ~loc extension
+  | EC.Signature_item -> Ast_builder.Default.psig_extension ~loc extension []
+  | EC.Structure_item -> Ast_builder.Default.pstr_extension ~loc extension []
+  | EC.Ppx_import ->
+      (* Insert the error in the type decl manifest *)
+      let ptype_manifest =
+        Some (Ast_builder.Default.ptyp_extension ~loc extension)
+      in
+      { original_node with ptype_manifest }
+
+let exn_to_extension exn =
+  let error = exn_to_loc_error exn in
+  let loc = Location.Error.get_location error in
+  let extension = Location.Error.to_extension error in
+  (extension, loc)
+
+let exn_to_error_extension context original_node exn =
+  let extension, loc = exn_to_extension exn in
+  wrap_extension ~loc context original_node extension
+
+let exn_to_stri exn =
+  let extension, loc = exn_to_extension exn in
+  Ast_builder.Default.pstr_extension ~loc extension []
+
+let exn_to_sigi exn =
+  let extension, loc = exn_to_extension exn in
+  Ast_builder.Default.psig_extension ~loc extension []
+
 let rec map_node_rec context ts super_call loc base_ctxt x ~embed_errors =
   let ctxt =
     Expansion_context.Extension.make ~extension_point_loc:loc ~base:base_ctxt ()
@@ -207,7 +270,8 @@ let rec map_node_rec context ts super_call loc base_ctxt x ~embed_errors =
       (try
          E.For_context.convert_res ts ~ctxt ext
          |> With_errors.of_result ~default:None
-       with exn when embed_errors -> (None, [ exn_to_loc_error exn ]))
+       with exn when embed_errors ->
+         With_errors.return (Some (exn_to_error_extension context x exn)))
       >>= fun converted ->
       match converted with
       | None -> super_call base_ctxt x
@@ -227,7 +291,8 @@ let map_node context ts super_call loc base_ctxt x ~hook ~embed_errors =
       (try
          E.For_context.convert_res ts ~ctxt ext
          |> With_errors.of_result ~default:None
-       with exn when embed_errors -> (None, [ exn_to_loc_error exn ]))
+       with exn when embed_errors ->
+         With_errors.return (Some (exn_to_error_extension context x exn)))
       >>= fun converted ->
       match converted with
       | None -> super_call base_ctxt x
@@ -261,7 +326,8 @@ let rec map_nodes context ts super_call get_loc base_ctxt l ~hook ~embed_errors
           (try
              E.For_context.convert_inline_res ts ~ctxt ext
              |> With_errors.of_result ~default:None
-           with exn when embed_errors -> (None, [ exn_to_loc_error exn ]))
+           with exn when embed_errors ->
+             With_errors.return (Some [ exn_to_error_extension context x exn ]))
           >>= function
           | None ->
               super_call base_ctxt x >>= fun x ->
@@ -350,7 +416,7 @@ let context_free_attribute_modification ~loc =
    of one element; it only has [@@deriving].
 *)
 let handle_attr_group_inline attrs rf ~items ~expanded_items ~loc ~base_ctxt
-    ~embed_errors =
+    ~embed_errors ~convert_exn =
   List.fold_left attrs ~init:(return [])
     ~f:(fun acc (Rule.Attr_group_inline.T group) ->
       acc >>= fun acc ->
@@ -368,10 +434,12 @@ let handle_attr_group_inline attrs rf ~items ~expanded_items ~loc ~base_ctxt
           try
             let expect_items = group.expand ~ctxt rf expanded_items values in
             return (expect_items :: acc)
-          with exn when embed_errors -> (acc, [ exn_to_loc_error exn ])))
+          with exn when embed_errors ->
+            let error_item = [ convert_exn exn ] in
+            return (error_item :: acc)))
 
-let handle_attr_inline attrs ~item ~expanded_item ~loc ~base_ctxt ~embed_errors
-    =
+let handle_attr_inline attrs ~convert_exn ~item ~expanded_item ~loc ~base_ctxt
+    ~embed_errors =
   List.fold_left attrs ~init:(return []) ~f:(fun acc (Rule.Attr_inline.T a) ->
       acc >>= fun acc ->
       Attribute.get_res a.attribute item |> of_result ~default:None
@@ -390,7 +458,9 @@ let handle_attr_inline attrs ~item ~expanded_item ~loc ~base_ctxt ~embed_errors
           try
             let expect_items = a.expand ~ctxt expanded_item value in
             return (expect_items :: acc)
-          with exn when embed_errors -> (acc, [ exn_to_loc_error exn ])))
+          with exn when embed_errors ->
+            let error_item = [ convert_exn exn ] in
+            return (error_item :: acc)))
 
 module Expect_mismatch_handler = struct
   type t = {
@@ -463,6 +533,15 @@ class map_top_down ?(expect_mismatch_handler = Expect_mismatch_handler.nop)
     |> sort_attr_inline |> Rule.Attr_inline.split_normal_and_expect
   in
 
+  let attr_str_class_decls, attr_str_class_decls_expect =
+    Rule.filter Attr_str_class_type_decl rules
+    |> sort_attr_group_inline |> Rule.Attr_group_inline.split_normal_and_expect
+  in
+  let attr_sig_class_decls, attr_sig_class_decls_expect =
+    Rule.filter Attr_sig_class_type_decl rules
+    |> sort_attr_group_inline |> Rule.Attr_group_inline.split_normal_and_expect
+  in
+
   let map_node = map_node ~hook ~embed_errors in
   let map_nodes = map_nodes ~hook ~embed_errors in
   let handle_attr_group_inline = handle_attr_group_inline ~embed_errors in
@@ -518,7 +597,8 @@ class map_top_down ?(expect_mismatch_handler = Expect_mismatch_handler.nop)
           | Some pattern -> (
               let generated_code =
                 try return (pattern e)
-                with exn when embed_errors -> (None, [ exn_to_loc_error exn ])
+                with exn when embed_errors ->
+                  return (Some (exn_to_error_extension EC.expression e exn))
               in
               generated_code >>= fun expr ->
               match expr with
@@ -532,7 +612,8 @@ class map_top_down ?(expect_mismatch_handler = Expect_mismatch_handler.nop)
           | Some pattern -> (
               let generated_code =
                 try return (pattern e)
-                with exn when embed_errors -> (None, [ exn_to_loc_error exn ])
+                with exn when embed_errors ->
+                  return (Some (exn_to_error_extension EC.expression e exn))
               in
               generated_code >>= fun expr ->
               match expr with
@@ -540,10 +621,12 @@ class map_top_down ?(expect_mismatch_handler = Expect_mismatch_handler.nop)
               | Some e -> self#expression base_ctxt e))
       | Pexp_constant (Pconst_integer (s, Some c)) -> (
           try expand_constant Integer c s
-          with exn when embed_errors -> (e, [ exn_to_loc_error exn ]))
+          with exn when embed_errors ->
+            return (exn_to_error_extension EC.expression e exn))
       | Pexp_constant (Pconst_float (s, Some c)) -> (
           try expand_constant Float c s
-          with exn when embed_errors -> (e, [ exn_to_loc_error exn ]))
+          with exn when embed_errors ->
+            return (exn_to_error_extension EC.expression e exn))
       | _ -> super#expression base_ctxt e
 
     (* Pre-conditions:
@@ -688,43 +771,57 @@ class map_top_down ?(expect_mismatch_handler = Expect_mismatch_handler.nop)
                     loop rest ~in_generated_code >>| fun rest -> items @ rest)
             | _ -> (
                 super#structure_item base_ctxt item >>= fun expanded_item ->
+                let convert_exn = exn_to_stri in
                 match (item.pstr_desc, expanded_item.pstr_desc) with
                 | Pstr_type (rf, tds), Pstr_type (exp_rf, exp_tds) ->
                     (* No context-free rule can rewrite rec flags atm, this
                        assert acts as a failsafe in case it ever changes *)
                     assert (Poly.(rf = exp_rf));
                     handle_attr_group_inline attr_str_type_decls rf ~items:tds
-                      ~expanded_items:exp_tds ~loc ~base_ctxt
+                      ~expanded_items:exp_tds ~loc ~base_ctxt ~convert_exn
                     >>= fun extra_items ->
                     handle_attr_group_inline attr_str_type_decls_expect rf
                       ~items:tds ~expanded_items:exp_tds ~loc ~base_ctxt
+                      ~convert_exn
                     >>= fun expect_items ->
                     with_extra_items expanded_item ~extra_items ~expect_items
                       ~rest ~in_generated_code
                 | Pstr_modtype mtd, Pstr_modtype exp_mtd ->
                     handle_attr_inline attr_str_module_type_decls ~item:mtd
-                      ~expanded_item:exp_mtd ~loc ~base_ctxt
+                      ~expanded_item:exp_mtd ~loc ~base_ctxt ~convert_exn
                     >>= fun extra_items ->
                     handle_attr_inline attr_str_module_type_decls_expect
                       ~item:mtd ~expanded_item:exp_mtd ~loc ~base_ctxt
+                      ~convert_exn
                     >>= fun expect_items ->
                     with_extra_items expanded_item ~extra_items ~expect_items
                       ~rest ~in_generated_code
                 | Pstr_typext te, Pstr_typext exp_te ->
                     handle_attr_inline attr_str_type_exts ~item:te
-                      ~expanded_item:exp_te ~loc ~base_ctxt
+                      ~expanded_item:exp_te ~loc ~base_ctxt ~convert_exn
                     >>= fun extra_items ->
                     handle_attr_inline attr_str_type_exts_expect ~item:te
-                      ~expanded_item:exp_te ~loc ~base_ctxt
+                      ~expanded_item:exp_te ~loc ~base_ctxt ~convert_exn
                     >>= fun expect_items ->
                     with_extra_items expanded_item ~extra_items ~expect_items
                       ~rest ~in_generated_code
                 | Pstr_exception ec, Pstr_exception exp_ec ->
                     handle_attr_inline attr_str_exceptions ~item:ec
-                      ~expanded_item:exp_ec ~loc ~base_ctxt
+                      ~expanded_item:exp_ec ~loc ~base_ctxt ~convert_exn
                     >>= fun extra_items ->
                     handle_attr_inline attr_str_exceptions_expect ~item:ec
-                      ~expanded_item:exp_ec ~loc ~base_ctxt
+                      ~expanded_item:exp_ec ~loc ~base_ctxt ~convert_exn
+                    >>= fun expect_items ->
+                    with_extra_items expanded_item ~extra_items ~expect_items
+                      ~rest ~in_generated_code
+                | Pstr_class_type cds, Pstr_class_type exp_cds ->
+                    handle_attr_group_inline attr_str_class_decls Nonrecursive
+                      ~items:cds ~expanded_items:exp_cds ~loc ~base_ctxt
+                      ~convert_exn
+                    >>= fun extra_items ->
+                    handle_attr_group_inline attr_str_class_decls_expect
+                      Nonrecursive ~items:cds ~expanded_items:exp_cds ~loc
+                      ~base_ctxt ~convert_exn
                     >>= fun expect_items ->
                     with_extra_items expanded_item ~extra_items ~expect_items
                       ~rest ~in_generated_code
@@ -783,43 +880,57 @@ class map_top_down ?(expect_mismatch_handler = Expect_mismatch_handler.nop)
                     loop rest ~in_generated_code >>| fun rest -> items @ rest)
             | _ -> (
                 super#signature_item base_ctxt item >>= fun expanded_item ->
+                let convert_exn = exn_to_sigi in
                 match (item.psig_desc, expanded_item.psig_desc) with
                 | Psig_type (rf, tds), Psig_type (exp_rf, exp_tds) ->
                     (* No context-free rule can rewrite rec flags atm, this
                        assert acts as a failsafe in case it ever changes *)
                     assert (Poly.(rf = exp_rf));
                     handle_attr_group_inline attr_sig_type_decls rf ~items:tds
-                      ~expanded_items:exp_tds ~loc ~base_ctxt
+                      ~expanded_items:exp_tds ~loc ~base_ctxt ~convert_exn
                     >>= fun extra_items ->
                     handle_attr_group_inline attr_sig_type_decls_expect rf
                       ~items:tds ~expanded_items:exp_tds ~loc ~base_ctxt
+                      ~convert_exn
                     >>= fun expect_items ->
                     with_extra_items expanded_item ~extra_items ~expect_items
                       ~rest ~in_generated_code
                 | Psig_modtype mtd, Psig_modtype exp_mtd ->
                     handle_attr_inline attr_sig_module_type_decls ~item:mtd
-                      ~expanded_item:exp_mtd ~loc ~base_ctxt
+                      ~expanded_item:exp_mtd ~loc ~base_ctxt ~convert_exn
                     >>= fun extra_items ->
                     handle_attr_inline attr_sig_module_type_decls_expect
                       ~item:mtd ~expanded_item:exp_mtd ~loc ~base_ctxt
+                      ~convert_exn
                     >>= fun expect_items ->
                     with_extra_items expanded_item ~extra_items ~expect_items
                       ~rest ~in_generated_code
                 | Psig_typext te, Psig_typext exp_te ->
                     handle_attr_inline attr_sig_type_exts ~item:te
-                      ~expanded_item:exp_te ~loc ~base_ctxt
+                      ~expanded_item:exp_te ~loc ~base_ctxt ~convert_exn
                     >>= fun extra_items ->
                     handle_attr_inline attr_sig_type_exts_expect ~item:te
-                      ~expanded_item:exp_te ~loc ~base_ctxt
+                      ~expanded_item:exp_te ~loc ~base_ctxt ~convert_exn
                     >>= fun expect_items ->
                     with_extra_items expanded_item ~extra_items ~expect_items
                       ~rest ~in_generated_code
                 | Psig_exception ec, Psig_exception exp_ec ->
                     handle_attr_inline attr_sig_exceptions ~item:ec
-                      ~expanded_item:exp_ec ~loc ~base_ctxt
+                      ~expanded_item:exp_ec ~loc ~base_ctxt ~convert_exn
                     >>= fun extra_items ->
                     handle_attr_inline attr_sig_exceptions_expect ~item:ec
-                      ~expanded_item:exp_ec ~loc ~base_ctxt
+                      ~expanded_item:exp_ec ~loc ~base_ctxt ~convert_exn
+                    >>= fun expect_items ->
+                    with_extra_items expanded_item ~extra_items ~expect_items
+                      ~rest ~in_generated_code
+                | Psig_class_type cds, Psig_class_type exp_cds ->
+                    handle_attr_group_inline attr_sig_class_decls Nonrecursive
+                      ~items:cds ~expanded_items:exp_cds ~loc ~base_ctxt
+                      ~convert_exn
+                    >>= fun extra_items ->
+                    handle_attr_group_inline attr_sig_class_decls_expect
+                      Nonrecursive ~items:cds ~expanded_items:exp_cds ~loc
+                      ~base_ctxt ~convert_exn
                     >>= fun expect_items ->
                     with_extra_items expanded_item ~extra_items ~expect_items
                       ~rest ~in_generated_code
