@@ -24,6 +24,7 @@ let pretty = ref false
 let styler = ref None
 let output_metadata_filename = ref None
 let corrected_suffix = ref ".ppx-corrected"
+let keywords = ref None
 
 let ghost =
   object
@@ -664,7 +665,7 @@ let map_structure_gen st ~tool_name ~hook ~expect_mismatch_handler ~input_name
   let cookies_and_check st =
     Cookies.call_post_handlers T;
     let errors =
-      if !perform_checks then (
+      if !perform_checks then
         (* TODO: these two passes could be merged, we now have more passes for
            checks than for actual rewriting. *)
         let unused_attributes_errors =
@@ -676,15 +677,15 @@ let map_structure_gen st ~tool_name ~hook ~expect_mismatch_handler ~input_name
           else []
         in
         let not_seen_errors = Attribute.collect_unseen_errors () in
-        (if !perform_locations_check then
-           let open Location_check in
-           ignore
-             ((enforce_invariants !loc_fname)#structure st
-                Non_intersecting_ranges.empty
-               : Non_intersecting_ranges.t));
-        unused_attributes_errors @ unused_extension_errors @ not_seen_errors)
+        unused_attributes_errors @ unused_extension_errors @ not_seen_errors
       else []
     in
+    (if !perform_locations_check then
+       let open Location_check in
+       ignore
+         ((enforce_invariants !loc_fname)#structure st
+            Non_intersecting_ranges.empty
+           : Non_intersecting_ranges.t));
     with_errors errors st
   in
   let file_path = get_default_path_str st in
@@ -740,7 +741,7 @@ let map_signature_gen sg ~tool_name ~hook ~expect_mismatch_handler ~input_name
   let cookies_and_check sg =
     Cookies.call_post_handlers T;
     let errors =
-      if !perform_checks then (
+      if !perform_checks then
         (* TODO: these two passes could be merged, we now have more passes for
            checks than for actual rewriting. *)
         let unused_attributes_errors =
@@ -752,15 +753,15 @@ let map_signature_gen sg ~tool_name ~hook ~expect_mismatch_handler ~input_name
           else []
         in
         let not_seen_errors = Attribute.collect_unseen_errors () in
-        (if !perform_locations_check then
-           let open Location_check in
-           ignore
-             ((enforce_invariants !loc_fname)#signature sg
-                Non_intersecting_ranges.empty
-               : Non_intersecting_ranges.t));
-        unused_attributes_errors @ unused_extension_errors @ not_seen_errors)
+        unused_attributes_errors @ unused_extension_errors @ not_seen_errors
       else []
     in
+    (if !perform_locations_check then
+       let open Location_check in
+       ignore
+         ((enforce_invariants !loc_fname)#signature sg
+            Non_intersecting_ranges.empty
+           : Non_intersecting_ranges.t));
     with_errors errors sg
   in
   let file_path = get_default_path_sig sg in
@@ -1154,8 +1155,8 @@ let process_file (kind : Kind.t) fn ~input_name ~relocate ~output_mode
           let ppf = Stdlib.Format.formatter_of_out_channel oc in
           let ast = add_cookies ast in
           (match ast with
-          | Intf ast -> Sexp.pp_hum ppf (Ast_traverse.sexp_of#signature ast)
-          | Impl ast -> Sexp.pp_hum ppf (Ast_traverse.sexp_of#structure ast));
+          | Intf ast -> Pp_ast.signature ppf ast
+          | Impl ast -> Pp_ast.structure ppf ast);
           Stdlib.Format.pp_print_newline ppf ())
   | Reconcile mode ->
       Reconcile.reconcile !replacements
@@ -1409,6 +1410,14 @@ let standalone_args =
     ( "-corrected-suffix",
       Arg.Set_string corrected_suffix,
       "SUFFIX Suffix to append to corrected files" );
+    ( "-keywords",
+      Arg.String (fun s -> keywords := Some s),
+      "<version+list> Set keywords according to the version+list \
+       specification. Allows using a set of keywords different from the one of \
+       the current compiler for backward compatibility." );
+    ( "--keywords",
+      Arg.String (fun s -> keywords := Some s),
+      "<version+list> Same as -keywords" );
   ]
 
 let get_args ?(standalone_args = standalone_args) () =
@@ -1418,6 +1427,7 @@ let standalone_main () =
   let usage = Printf.sprintf "%s [extra_args] [<files>]" exe_name in
   let args = get_args () in
   Arg.parse (Arg.align args) set_input usage;
+  Astlib.Keyword.apply_keyword_edition ~cli:!keywords ();
   interpret_mask ();
   if !request_print_transformations then (
     print_transformations ();
